=begin
UTILISATION DE LA BASE DE DONNEES SQLITE3

. Cration de tables
. Insertion/modif de valeurs
. Lecture avec paramtres de slection
. Traitement par enreg ou par champs associatifs
. Diverses syntaxes de lecture
=end


require 'sqlite3'



#####   OUVERTURE DE LA BASE DE DONNEES
db = SQLite3::Database.open "x:\\...\\bibli.db"

#####   CREATION DES TABLES
db.execute ( \
  "create table if not exists auteurs ( \
	autnum int not null, \
	autabr varchar(15) not null, \
	autnom char(30) not null, \
	autnbl int default 0, \
	constraint cleaut primary key (autnum) \
   ) ;" \
)

db.execute ( \
  "create table if not exists livres ( \
	livnum int not null, \
	livaut int not null, \
	livtit varchar(50) not null, \
	constraint cleliv primary key (livnum) \
   );" \
)



#####   AJOUT DES DONNEES
# auteurs
db.execute("delete from auteurs;")
db.execute("insert into auteurs (autnum, autabr, autnom) values(1, 'VERNE', 'Robert VERNE');")
db.execute("insert into auteurs (autnum, autabr, autnom) values(2, 'BUZZATI', 'Dino BUZZATI');")
db.execute("insert into auteurs values(3, 'ZOLA', 'Emile ZOLA', 0);")
db.execute("update auteurs set autnom='Jules VERNE' where autnum=1;")

# livres 
db.execute("delete from livres;")
db.execute("insert into livres values(1, 1, '20000 lieues sous les mers');")
db.execute("insert into livres values(2, 1, 'Le tour du monde en 80 jours');")
db.execute("insert into livres values(3, 3, 'Germinal');")



#####   LECTURE DES DONNEES
# Plusieurs modes de lecture

###  -----   PREPARE + EXECUTE + NEXT + ENREG COMPLET (pas de sparation des champs)

puts "---   Methode 1"
qry = db.prepare "SELECT * FROM livres inner join auteurs on autnum=livaut "
rs = qry.execute
while (lec = rs.next) do
    puts lec.join "\s" # 1 enreg par ligne
end
qry.close # fermer la requete sinon erreur sur fermeture db
puts "\n"

###  -----   PREPARE + EXECUTE + EACH + ENREG COMPLET

puts "---   Methode 2"
qry = db.prepare "SELECT * FROM livres inner join auteurs on autnum=livaut where autabr='VERNE'"
rs = qry.execute
rs.each do |lec|
    puts lec << "\s" # 1 ligne par champ + ligne blanche entre deux enregs
end
qry.close # fermer la requete sinon erreur sur fermeture db
puts "\n"

###  -----   EXECUTE + FOR + ENREG COMPLET

puts "---   Methode 3"
tout = db.execute("SELECT * FROM livres inner join auteurs on autnum=livaut where autabr='VERNE'")
for lec in tout do
    puts lec.join "\s"
end
puts "\n"

###  -----   EXECUTE + EACH + CHAMPS ASSOCIATIFS

puts "---   Methode 4"
db.results_as_hash = true
tout = db.execute("SELECT * FROM livres inner join auteurs on autnum=livaut where autabr='VERNE'")
tout.each do |lec|
    puts lec["autabr"] << "\t" << lec["livtit"] << "\s"
end
db.results_as_hash = false
puts "\n"

###  -----   PREPARE + PARAMETRE + EXECUTE + NEXT + ENREG COMPLET

puts "---   Methode 5"
qry = db.prepare "SELECT * FROM livres inner join auteurs on autnum=livaut where autabr=?"
rs = qry.execute "ZOLA"
while (lec = rs.next) do
    puts lec.join "\s" # 1 enreg par ligne
end
puts "\n"

rs = qry.execute "VERNE"
while (lec = rs.next) do
    puts lec.join "\s" # 1 enreg par ligne
end
puts "\n"

qry.close # fermer la requete sinon erreur sur fermeture db

###  -----   PREPARE + PARAMETRE + EXECUTE + EACH + CHAMPS ASSOCIATIFS

puts "---   Methode 6"
db.results_as_hash = true
qry = db.prepare "SELECT * FROM livres inner join auteurs on autnum=livaut where autabr=?"
rs = qry.execute "ZOLA"
rs.each do |lec| 
    puts lec["livtit"] << "\s" # 1 enreg par ligne
end
puts "\n"

# pas besoin de reprparer la requete
rs = qry.execute "VERNE"
rs.each do |lec| 
    puts lec["livtit"] << "\s" # 1 enreg par ligne
end
puts "\n"
db.results_as_hash = false

qry.close # fermer la requete sinon erreur sur fermeture db

###  -----   PREPARE + PARAMETRE + EXECUTE + EACH + CHAMPS ASSOCIATIFS + BIND
# 2 syntaxes bind_param et bind_paramS
# La meme variable qry, utilise pour deux prepare, cre deux handles qui empecheront la fermeture de la base
# (objet non finalis). Il faut donc la fermer  chaque fois

puts "---   Methode 7"
db.results_as_hash = true
qry = db.prepare "SELECT * FROM livres inner join auteurs on autnum=livaut where autabr=:abr"
abr1="ZOLA"
qry.bind_param 1, abr1 
abr1="ZOLA"
rs = qry.execute
rs.each do |lec| 
    puts lec["livtit"] << "\s" # 1 enreg par ligne
end
puts "\n"
qry.close

# il faut reprparer la requete, refaire le bind, le changement de valeur ne suffit pas
qry = db.prepare "SELECT * FROM livres inner join auteurs on autnum=livaut where autabr=:abr"
abr2="VERNE"
qry.bind_params abr2
rs = qry.execute
rs.each do |lec| 
    puts lec["livtit"] << "\s" # 1 enreg par ligne
end
puts "\n"
db.results_as_hash = false

qry.close # fermer la requete sinon erreur sur fermeture db



###  -----   EXECUTE + PARAMETRE + EACH + CHAMPS ASSOCIATIFS

puts "---   Methode 8"
db.results_as_hash = true
tout = db.execute "SELECT * FROM livres inner join auteurs on autnum=livaut where autabr=?", "VERNE"
tout.each do |lec|
    puts lec["autabr"] << "\t" << lec["livtit"] << "\s"
end
db.results_as_hash = false
puts "\n"

db.results_as_hash = true
tout = db.execute "SELECT * FROM livres inner join auteurs on autnum=livaut where autabr=:abr" , "ZOLA"
tout.each do |lec|
    puts lec["autabr"] << "\t" << lec["livtit"] << "\s"
end
db.results_as_hash = false
puts "\n"



#####   FERMETURE DE LA BASE
db.close
